program addtime;

{$APPTYPE CONSOLE}

uses
  SysUtils, DateUtils, Classes, Math;

var
  fileinfo: tsearchrec;
  add: boolean;
  curfile: tfilestream;
  filemask: string;
  fileattr: byte;
  recurse: boolean;
  i: byte;
  startpath: string;
  curdir: string;
  olddate: tdatetime;
  newdate: tdatetime;
  files_ok: longword;
  files_bad: longword;
  verbose: boolean;

procedure givehelp;
  begin
    writeln;
    writeln('AddTime 1.21, 13 Oct 2002, by Roman Korcek <thehacker@host.sk>');
    writeln('Adds or subtracts time from filedates');
    writeln;
    writeln('Usage:');
    writeln('addtime <+|-> <years months days hours minutes seconds> [r] [v] [file_mask]');
    writeln;
    writeln('         + or -  + means add time, - means subtract ;-)');
    writeln('years - seconds: Provide a number for each, a 0 for unneeded values');
    writeln('              r: Work recursively, ie change the time of all files');
    writeln('                 specified by file_mask in all subdirectories');
    writeln('              v: Verbose mode - prints the names of all processed files,');
    writeln('                 not only the ones that gave errors');
    writeln('      file_mask: Specify a file mask to use. Default is *.*');
    writeln;
    writeln('Examples:');
    writeln('addtime + 0 0 0 2 8 0');
    writeln('Changes the time of all the files in the current directory by + 2 hours,');
    writeln('8 minutes');
    writeln;
    writeln('addtime - 2 0 0 0 0 0 r v *.jpg');
    writeln('Changes the time of all *.jpg files in all subdirectories by minus 2 years');
    writeln('and lists them all');
    halt;
  end;

function namewithpath: string;
  begin
    if getcurrentdir = startpath then
     result := (copy(getcurrentdir, length(startpath) + 1, length(getcurrentdir))
      + fileinfo.name)
    else
     result := (copy(getcurrentdir, length(startpath) + 2, length(getcurrentdir)) + '\'
      + fileinfo.name);
  end;

procedure changetimes;
  begin                                                        // check if there are
    if findfirst(filemask, fileattr, fileinfo) = 0 then        // any files to change
      begin
        repeat
          if (fileinfo.name <> 'addtime.exe') and (fileinfo.name <> '.')  // filter out nonsense
           and (fileinfo.name <> '..') then
             begin
               olddate := filedatetodatetime(fileinfo.time); // get the filedate


               // increase the date by the amount specified by the respective parameter

               newdate := olddate;
               newdate := incyear(newdate, ifthen(add, strtoint(paramstr(2)), -(strtoint(paramstr(2)))));
               newdate := incmonth(newdate, ifthen(add, strtoint(paramstr(3)), -(strtoint(paramstr(3)))));
               newdate := incday(newdate, ifthen(add, strtoint(paramstr(4)), -(strtoint(paramstr(4)))));
               newdate := inchour(newdate, ifthen(add, strtoint(paramstr(5)), -(strtoint(paramstr(5)))));
               newdate := incminute(newdate, ifthen(add, strtoint(paramstr(6)), -(strtoint(paramstr(6)))));
               newdate := incsecond(newdate, ifthen(add, strtoint(paramstr(7)), -(strtoint(paramstr(7)))));


               // try to set the date of the file to the new date

               curfile := tfilestream.create(fileinfo.name, fmopenwrite);
               if filesetdate(curfile.handle, datetimetofiledate(newdate)) = 0 then
                 begin
                   inc(files_ok);    // increase the number of correctly processed files by 1
                   if verbose = true then
                     begin
                       writeln;


                       // namewithpath returns the filename with the path in front of the name
                       // if we are not in the starting (root) path (dir)

                       writeln(namewithpath);
                       writeln(datetimetostr(olddate), ' -> ', datetimetostr(newdate));
                     end;
                 end
               else
                 begin
                   inc(files_bad);


                   // an empty line to make it look better when in verbose mode

                   if verbose = true then writeln;
                   writeln('ERROR: ', namewithpath);
                 end;
               curfile.free;
             end;
        until findnext(fileinfo) <> 0;
        findclose(fileinfo);
      end;
  end;

procedure processdir;
  begin
    changetimes;             // first, let's change the dates of the files in the current dir
    if recurse = true then
      begin


        // let's find the first subdir

        findfirst('*.*', $10, fileinfo);
        findnext(fileinfo);
        repeat
          i := findnext(fileinfo);
        until (i <> 0) or (fileinfo.attr and faDirectory > 1);
        findclose(fileinfo);


        // i determines if subdir was found. 0 means subdir was found, any other value
        // means no subdir found

        // let's keep going deeper and deeper into a subdir and processing each subdir
        // until no further subdir is found (subdir subdir subdir subdir!)

        if i = 0 then
          begin
            chdir(fileinfo.name);
            processdir;
          end


        // if no deeper subdir is found then let's find out the name of the current dir,
        // go one level higher and see if there are more dirs than the one we were in.
        // If yes, then let's go into them and process them
        // We repeat this until we're not back up in the root (start) path

        else
          if getcurrentdir <> startpath then
            begin
              repeat
                curdir := copy(getcurrentdir, lastdelimiter('\', getcurrentdir) + 1,
                 length(getcurrentdir) - lastdelimiter('\', getcurrentdir));
                chdir('..');
                findfirst('*.*', $10, fileinfo);
                findnext(fileinfo);
                repeat
                  findnext(fileinfo);
                until (fileinfo.attr and faDirectory > 1) and (fileinfo.Name = curdir);
                repeat
                  i := findnext(fileinfo);
                until (i <> 0) or (fileinfo.attr and faDirectory > 1);
              until (i = 0) or (getcurrentdir = startpath);
              findclose(fileinfo);

              if i = 0 then                                 // when subdir is found let's
                begin                                       // process it, otherwise end it
                  chdir(fileinfo.name);                     // (and possibly return to where
                  processdir;                               // we were called recursively)
                end;
            end;
      end;
end;


begin
  if (paramcount < 7) or (paramcount > 10) then  // the number of parameters must be between
   givehelp;                                     // 7 and 10
  if paramstr(1) = '+' then
   add := true
  else
   if paramstr(1) = '-' then
    add := false
   else
    givehelp;

  filemask := '*.*';                         // set filemask to default *.*
  fileattr := $27;                           // set file attributes to exclude dirs as default
  recurse := false;                          // no recursion on default

  for i := 8 to paramcount do                // check for the 'r' and 'd' parameters,
    begin                                    // modify variables accordingly
      if paramstr(i) = 'r' then
       recurse := true;
      if paramstr(i) = 'v' then
       verbose := true;
//      if paramstr(i) = 'd' then            // d isn't currently used
//       fileattr := $37;
    end;

  i := 7;                              // 7 is the minimal number of parameters.
  if recurse = true then               // Add 1 to i for every additional parameter.
   inc(i);                             // If there is a parameter we haven't used yet
  if verbose = true then               // it must be the filemask, so assign it.
   inc(i);
//  if fileattr = $37 then
//   inc(i);
  if paramcount > i then
   filemask := paramstr(paramcount);

  startpath := getcurrentdir;          // set the root path so we know where to stop when
                                       // we return from subdirs

  files_ok := 0;
  files_bad := 0;

  processdir;                          // let's start! ;-)


  // and let's print the final statistics...

  writeln;
  writeln(' Total: ', files_ok + files_bad);
  writeln('    OK: ', files_ok);
  writeln('Failed: ', files_bad);

end.
